package org.capcaval.ccoutils.lafabrique.command;

import java.io.IOException;
import java.io.Writer;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;

import org.capcaval.ccoutils.common.CommandResult;
import org.capcaval.ccoutils.file.FileTools;
import org.capcaval.ccoutils.file.TokenTool;
import org.capcaval.ccoutils.lafabrique.AbstractProject;
import org.capcaval.ccoutils.lang.ArrayTools;
import org.capcaval.ccoutils.lang.ClassTools;

public class CommandNewApplication {

	public static CommandResult newApplication(AbstractProject proj, String fullClassName) {
		CommandResult cr = new CommandResult();

		try {

			String[] packageArray = fullClassName.split("\\.");
			
			String className = fullClassName;
			String packageName = "";
			
			// get last
			if(packageArray.length>1){
				className = packageArray[packageArray.length - 1];
				packageName = fullClassName.substring(0, fullClassName.length() - className.length() - 1);
				packageName = "package " + packageName + ";";
				}

			String templateStr = FileTools.getLocalTextFile("template.application");

			Map<String, String> map = ArrayTools.newMap(
					"appName", className, 
					"authorName", proj.authorList[0] != null ? proj.authorList[0] : "", 
					"packageName", packageName, 
					"className", className);

			String appClass = TokenTool.replaceTokenFromReader(templateStr, map, '{', '}');

			String[] multiLineStr = appClass.split("\n");

			FileTools.writeFile(proj.sourceDirList[0].resolve(ClassTools.getFileNameFromClassName(fullClassName)),
					multiLineStr);
			
			// create the script for windows and linux
			String batStr = FileTools.getLocalTextFile("template.laFab.bat");

			StringBuilder libStr = new StringBuilder(proj.binDirPath.toString()+";");
			String libPathStr = proj.libDirList.get(0).toString();
			for(String lib : proj.libList){
				libStr.append(libPathStr + "/" + lib);
				libStr.append(";");
			}
			
			String classPathStr = libStr.toString();
			if(proj.jar.name != ""){
				classPathStr = classPathStr + proj.productionDirPath.toString() + "/" + proj.jar.name;
			}
			if(proj.libList.length>0){
				classPathStr = "-cp " + classPathStr;
			}
			
			if(classPathStr.charAt(classPathStr.length()-1) == ';'){
				classPathStr = classPathStr.substring(0, classPathStr.length()-1);
			}
			
			map = ArrayTools.newMap(
					"appClass", fullClassName,  
					"librairieList", classPathStr);

			batStr = TokenTool.replaceTokenFromReader(batStr, map, '{', '}');

			FileTools.writeFile("./" + className + ".bat", batStr);
			
			// linux
			String shStr = FileTools.getLocalTextFile("template.laFab.sh");
			classPathStr = classPathStr.replace(";", ":");
			
			map = ArrayTools.newMap(
					"appClass", fullClassName, 
					"librairieList", classPathStr);

			shStr = TokenTool.replaceTokenFromReader(shStr, map, '{', '}');

			Path p = FileTools.writeFile("./" + className, shStr);
			p.toFile().setExecutable(true);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return cr;
	}

}
