package org.capcaval.ccoutils.lafabrique.command;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.List;

import org.capcaval.ccoutils.common.CommandResult;
import org.capcaval.ccoutils.file.FileTools;
import org.capcaval.ccoutils.file.FileTools.FilePropertyEnum;
import org.capcaval.ccoutils.lafabrique.DefaultConfigProject;
import org.capcaval.ccoutils.lang.ArrayTools;
import org.capcaval.ccoutils.lang.StringMultiLine;

public class CommandNewProject {

	public static CommandResult newProject(String name, Path projectPath, DefaultConfigProject defaultConfigProject) {
		CommandResult cr = new CommandResult();
		
		List<Path> subDirList = ArrayTools.newArrayList(defaultConfigProject.projectDir);
		// add all the source directory
		for(Path srcPath : defaultConfigProject.sourceDirList){
			subDirList.add(srcPath);
		}
		// add all the lib directory
		for(Path libPath : defaultConfigProject.libDirList){
			subDirList.add(libPath);
		}
		
		StringMultiLine returnedMessage = new StringMultiLine("[laFabrique] INFO : create all following sub-dirs at "
				+ projectPath.toString());

		try {
			for (Path dir : subDirList) {
				Path p = Paths.get(projectPath.toString(), dir.toString());

				if (p.toFile().exists() == true) {
					System.out.print("Directory " + p + " already exist, do you want to overwrite it ? ");
					String str = System.console().readLine();
					if (str.contains("y")) {
						Files.createDirectories(p);
					}
				} else {
					Files.createDirectories(p);
					returnedMessage.addLine("[laFabrique] INFO : " + p + " is created");
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
			returnedMessage.addLine("[laFabrique] ERROR : can not create the directory : " + projectPath.toString());
		}

		try {
			returnedMessage.addLine(CommandNewProject.class.getResource("laFab").toString());
		} catch (Exception e) {
			e.printStackTrace();
		}

		// copy the ccoutils lib
		// to do verify that the jar is existing otherwise get it inside 20_prod
		if (Files.exists(Paths.get("02_lib/ccOutils.jar"))) {
			copyFile(returnedMessage, "02_lib/ccOutils.jar", projectPath.toString() + "/02_lib/ccOutils.jar");
		} else if (Files.exists(Paths.get("20_prod/ccOutils.jar"))) {
			copyFile(returnedMessage, "20_prod/ccOutils.jar", projectPath.toString() + "/02_lib/ccOutils.jar");
		} else {
			returnedMessage
					.addLine("[laFabrique] ERROR : can not find ccOutils.jar inside 02_lib or inside 20_prod directories.");
		}

		InputStream laFabStream = CommandNewProject.class.getResourceAsStream("laFab");
		FileTools.saveInputStream(laFabStream, Paths.get(projectPath.toString(), "laFab"), FilePropertyEnum.xr_);

		InputStream laFabBatStream = CommandNewProject.class.getResourceAsStream("laFab.bat");
		FileTools.saveInputStream(laFabBatStream, Paths.get(projectPath.toString(), "laFab.bat"), FilePropertyEnum.xr_);

		// create a new Proj
		try {
			// Path templatePath =
			InputStream prjStream = CommandNewProject.class.getResourceAsStream("Project.java.template");

			InputStreamReader prjReader = new InputStreamReader(prjStream);

			FileTools.replaceTokenInFile(prjReader, Paths.get(projectPath.toString() + "/00_prj/prj/", name + ".java"),
					ArrayTools.newMap("projectName", name), '{', '}');

		
			// create the default project
			InputStream defPrjStream = CommandNewProject.class.getResourceAsStream("template.defaultProject");

			InputStreamReader defPrjReader = new InputStreamReader(defPrjStream);

			FileTools.replaceTokenInFile(defPrjReader, Paths.get(projectPath.toString() + "/00_prj/prj/DefaultProject.java"),
					ArrayTools.newMap("defaultProjectClass", name), '{', '}');

		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return cr;
	}
	
	public static void copyFile(StringMultiLine returnedMessage, String srcFile, String destDir) {
		try {
			// copy the file
			Files.copy(Paths.get(srcFile), Paths.get(destDir), StandardCopyOption.REPLACE_EXISTING);
			returnedMessage.addLine("[laFabrique] INFO : " + Paths.get(destDir + "/" + srcFile).toFile().toString()
					+ " is added");

		} catch (IOException e) {
			returnedMessage.addLine("[laFabrique] ERROR : can not copy the file : " + Paths.get(srcFile) + " to "
					+ Paths.get(destDir + "/" + srcFile));
			returnedMessage.addLine(e.toString());
			if (e.getCause() != null) {
				returnedMessage.addLine(e.getCause().toString());
			}
		}
	}

}
